package com.massa.util.extensions.nativeproperty.compiler;

import java.util.HashMap;
import java.util.Map;

/**
 * ClassLoader allowing to load dynamically compiled code.
 */
public class DynamicClassLoader extends ClassLoader {

	private final Map<String, CompiledCode> customCompiledCode = new HashMap<String, CompiledCode>();

	public DynamicClassLoader(final ClassLoader parent) {
		super(parent);
	}

	public void setCode(final CompiledCode cc) {
		this.customCompiledCode.put(cc.getName(), cc);
	}

	@Override
	protected Class<?> findClass(final String name) throws ClassNotFoundException {
		final CompiledCode cc = this.customCompiledCode.get(name);
		if (cc == null) {
			return super.findClass(name);
		}
		final byte[] byteCode = cc.getByteCode();
		return this.defineClass(name, byteCode, 0, byteCode.length);
	}
}
