package com.massa.util.extensions.nativeproperty;

import org.junit.Assert;
import org.junit.Test;

import com.massa.util.UtilsException;
import com.massa.util.property.IFieldAccessor;
import com.massa.util.property.IMethodInvoker;
import com.massa.util.property.NoCacheArrayProvider;
import com.massa.util.property.ReflectionFieldAccessor;
import com.massa.util.property.ReflectionMethodInvoker;

public class NativePropertyTest {
	@Test
	public void testFieldAccess() throws Exception {
		final NativeAccessBean bean = new NativeAccessBean();
		bean.field = "Hello!";
		final IFieldAccessor accessor = NativePropertyAccessCache.getInstance().getStrictField(bean, "field");
		Assert.assertEquals("Hello!", accessor.get(bean));
		accessor.set(bean, "Bye Bye!");
		Assert.assertEquals("Bye Bye!", bean.field);

		final IFieldAccessor prvaccessor = NativePropertyAccessCache.getInstance().getStrictField(bean, "privField");
		prvaccessor.set(bean, 24);
		Assert.assertEquals(24, prvaccessor.get(bean));
		
		Assert.assertTrue("Instance of failed: " + accessor.getClass(), accessor.getClass().getSimpleName().startsWith("$MRulesNativeAccess$"));
		Assert.assertTrue("Instance of failed: " + prvaccessor.getClass(), prvaccessor instanceof ReflectionFieldAccessor);
	}
	@Test
	public void testMethodInvocation() throws Exception {
		final NativeAccessBean bean = new NativeAccessBean();
		bean.setPrivField(0);
		IMethodInvoker getter = NativePropertyAccessCache.getInstance().getReadMethod(bean, "privField", Void.class);
		Object get = getter.invoke(NoCacheArrayProvider.INSTANCE, bean);
		Assert.assertEquals(Integer.valueOf(0), get);
		IMethodInvoker setter = NativePropertyAccessCache.getInstance().getWriteMethod(bean, "privField", Void.class);
		Object set = setter.invoke(NoCacheArrayProvider.INSTANCE, bean, Integer.valueOf(1));
		Assert.assertNull(set);
		Assert.assertEquals(Integer.valueOf(1), bean.getPrivField());

		try {
			getter.invoke(NoCacheArrayProvider.INSTANCE, bean, "Toto");
		} catch (final UtilsException e) {
		}

		try {
			setter.invoke(NoCacheArrayProvider.INSTANCE, bean);
		} catch (final UtilsException e) {
		}

		Assert.assertTrue("Instance of failed: " + getter.getClass(), getter.getClass().getSimpleName().startsWith("$MRulesNativeAccess$"));
		Assert.assertTrue("Instance of failed: " + setter.getClass(), setter.getClass().getSimpleName().startsWith("$MRulesNativeAccess$"));

		setter = NativePropertyAccessCache.getInstance().getWriteMethod(bean, "fullPrivField", Void.class);
		set = setter.invoke(NoCacheArrayProvider.INSTANCE, bean, Integer.valueOf(18));
		Assert.assertNull(set);
		getter = NativePropertyAccessCache.getInstance().getReadMethod(bean, "fullPrivField", Void.class);
		get = getter.invoke(NoCacheArrayProvider.INSTANCE, bean);
		Assert.assertEquals(Integer.valueOf(18), get);

		final IMethodInvoker privMethod = NativePropertyAccessCache.getInstance().getStrictMethod(bean, "privMethod", Void.class);
		
		final Object privMethodRes = privMethod.invoke(NoCacheArrayProvider.INSTANCE, bean);
		Assert.assertEquals("OK", privMethodRes);

		try {
			privMethod.invoke(NoCacheArrayProvider.INSTANCE, bean, "Toto");
		} catch (final UtilsException e) {
		}
		
		Assert.assertTrue("Instance of failed: " + getter.getClass(), privMethod instanceof ReflectionMethodInvoker);
		Assert.assertTrue("Instance of failed: " + setter.getClass(), privMethod instanceof ReflectionMethodInvoker);
		Assert.assertTrue("Instance of failed: " + privMethod.getClass(), privMethod instanceof ReflectionMethodInvoker);
	}
}
